package com.docus.demo.config;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = DbSqlserverDpConfig.PACKAGE, sqlSessionFactoryRef = "sqlserver1SqlSessionFactory")
public class DbSqlserverDpConfig {
    // 这里一定要指定精准  否则后果不堪设想
    static final String PACKAGE = "com.docus.demo.mapper.sqlserver1";
    static final String MAPPER_LOCATION = "classpath:mapper/sqlserver1/*.xml";

    @Bean(name = "sqlserver1DataSource")
    @ConfigurationProperties(prefix = "spring.datasource.dynamic.datasource.sqlserver1")
    public DataSource secondDataSource(){
        return new DruidDataSource();
    }

    /*注入事务*/
    @Bean(name = "sqlserver1TransactionManager")
    public DataSourceTransactionManager secondTransactionManager() {
        return new DataSourceTransactionManager(secondDataSource());
    }

    @Bean(name = "sqlserver1SqlSessionFactory")
    public SqlSessionFactory secondSqlSessionFactory(@Qualifier("sqlserver1DataSource") DataSource secondDataSource)
            throws Exception {
        final SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
        sessionFactory.setDataSource(secondDataSource);
        sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver()
                .getResources(DbSqlserverDpConfig.MAPPER_LOCATION));
        return sessionFactory.getObject();
    }

}
