package com.docus.demo.utils;

import lombok.extern.slf4j.Slf4j;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

@Slf4j
public class MyImageUtils {

    private static final MyImageUtils instance = new MyImageUtils();

    public static MyImageUtils getInstance() {
        return instance;
    }


    public boolean tiffToJpg(String inputFile, String outFile,int angle) {

        createParentRoot(outFile);
        try {
            BufferedImage bufferedImage = ImageIO.read(new File(inputFile));
            if (angle!=0){
                bufferedImage = rotateImage(bufferedImage, angle);
            }
            ImageIO.write(bufferedImage, "jpg", new File(outFile));

        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private static boolean createParentRoot(String outFile) {
        //判断父级目录是否存在 不存在需要创建
        File file = new File(outFile);
        String parentPath = file.getParent();
        File dir = new File(parentPath);
        if (!dir.exists()) {
            boolean b = dir.mkdirs();
            if (!b) {
                log.info("created parent dir fail :" + outFile);
                return true;
            }
        }
        return false;
    }

    private static BufferedImage rotateImage(BufferedImage image, int angle) {
        int w = image.getWidth();
        int h = image.getHeight();

        BufferedImage newImage = new BufferedImage(h, w, image.getType());
        Graphics2D g2 = newImage.createGraphics();
        g2.rotate(Math.toRadians(angle), h / 2, h / 2);
        g2.drawImage(image, null, 0, 0);
        g2.dispose();

        return newImage;
    }

}
